import argparse
import collections
import logging
import re
import typing as typ
from datetime import timedelta
from functools import partial
from pathlib import Path

import pandas as pd

TF_TD = {
    'M1' : timedelta(minutes=1), 'M5': timedelta(minutes=5), 'M15': timedelta(minutes=15),
    'M30': timedelta(minutes=30), 'H1': timedelta(hours=1), 'H4': timedelta(hours=4),
    'D1' : timedelta(days=1), 'W1': timedelta(weeks=1),
}
TD_TF = {v: k for k, v in TF_TD.items()}


def resample_ticks(tick_files_path: typ.Union[str, Path],
                   timeframes: typ.Iterable[typ.Union[timedelta, str]],
                   **kwargs
                   ) -> typ.List[Path]:
    path = Path(tick_files_path)
    f_re = re.compile(kwargs.get('file_pattern', r'^([A-Z]{6})-\d{4}-\d{2}'))
    tick_files = collections.defaultdict(list)
    logging.debug('Searching for tick files...')
    for file in path.glob('**/*.csv'):
        match = f_re.match(file.stem)
        if match:
            symbol = match.group(1)
            tick_files[symbol].append(file)
            logging.debug(f'{file.name} FOUND')
    for list_ in tick_files.values():
        list_.sort(key=lambda fp: fp.stem)
    read_csv = partial(pd.read_csv, names=['Symbol', 'Time', 'Bid', 'Ask'], index_col=1, parse_dates=True)
    res_paths = []
    if not tick_files:
        logging.warning(f'No tick files found in {path}')
    for symbol, file_list in tick_files.items():
        df = pd.concat(read_csv(f) for f in file_list)
        logging.debug(f're-sampling {symbol}')
        for timeframe in timeframes:
            job_path = path / f'{symbol}_{TD_TF.get(timeframe, timeframe)}.csv'
            ohlc_df = df['Bid'].resample(timeframe).ohlc()
            ohlc_df['Volume'] = df['Bid'].resample(timeframe).count()
            ohlc_df.to_csv(job_path)
            res_paths.append(job_path)
            logging.debug(f'saved {symbol} as {job_path}')
    return res_paths


def main():
    parser = argparse.ArgumentParser(description='Convert tickdata to OHLC')
    parser.add_argument('--path', '-p', default=None,
                        help=('Absolute path to the directory containing the tick-data CSV files. '
                              'If no path is specified then the current working directory is used.'))
    parser.add_argument('--timeframe', '-tf', action='append',
                        help='The timeframe for OHLC. [M1, M5, M15, M30, H1, H4, D1, W1]')
    parser.add_argument('--log', action='store_true', help='Save loggin data to a file in the search path.')
    args = parser.parse_args()
    logging.basicConfig(
        format='%(asctime)s\t%(message)s',
        level=logging.DEBUG,
        filename='ticks_to_ohlc.log' if args.log else None)
    logging.debug(f'args={vars(args)}')
    path = args.path or Path.cwd()
    timeframes = [TF_TD.get(t.upper(), t) for t in args.timeframe]
    resample_ticks(path, timeframes)


if __name__ == '__main__':
    main()
